"use strict";

const fs = require('fs');
const tls = require('tls');

const serverKey = fs.readFileSync('./certificates/alice.key')
const serverCert = fs.readFileSync('./certificates/alice.crt')
const caCert = fs.readFileSync('./certificates/ca.crt')

sendToSecurePair()

function sendToSecurePair() {
  // create server pair
  const serverConf = {
    key: serverKey,
    cert: serverCert + caCert,
    ca: [caCert],
    requestCert: false,
    rejectUnauthorized: false,
  }
  const serverCtxt = tls.createSecureContext(serverConf);
  const serverPair = tls.createSecurePair(serverCtxt, true, true, false);
  // create client pair
  const clientConf = {
    host: 'localhost',
    port: 4430,
    checkServerIdentity: function() {return undefined;}
  }
  const clientCtxt = tls.createSecureContext(clientConf);
  const clientPair = tls.createSecurePair(clientCtxt, false, true, false);

  // set event handlers
  clientPair.on('error', error => console.error('Client error: %s', error));
  clientPair.encrypted.on('data', data => {
    console.info("Encrypted bytes %s", data.length);
    // with setImmediate() crashes; otherwise continues
    //serverPair.destroy();
    setImmediate(() => serverPair.destroy());
  });
  serverPair.cleartext.on('data', data => {
    console.info("Received %s", data);
  });

  // attach one to the other
  clientPair.encrypted.pipe(serverPair.encrypted);
  serverPair.encrypted.pipe(clientPair.encrypted);

  // write to client
  clientPair.cleartext.write("hello world");
}

